//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "Types.h"
#include <Iphlpapi.h>
#include "loggable.h"

// This class has two purposes:
//
// Collect all statistics relevant to the upload/download bandwidth
// Provide an interface to the LAN adapter (Ethernet, slip, etc...)

class CBandWidthControl : public CLoggable
{
public:
	CBandWidthControl();
	~CBandWidthControl();

	// Update the history list. Must be called every cycle
	void Process();

	// Actualize current datarate values (Data+Control)
	// Remark: the overhead for the IP/TCP/UDP headers is not included
	void AddeMuleOut(uint32 octets) {m_statistic.eMuleOutOctets += octets;}
	void AddeMuleIn(uint32 octets) {m_statistic.eMuleInOctets += octets;}

#ifndef INCLUDE_IP_OVERHEAD 
	void AddeMuleOutUDPOverall(uint32 octets) {m_statistic.eMuleOutOverallOctets += octets;}
	void AddeMuleOutTCPOverall(uint32 octets) {m_statistic.eMuleOutOverallOctets += octets;}
	void AddeMuleInUDPOverall(uint32 octets) {m_statistic.eMuleInOverallOctets += octets;}
	void AddeMuleInTCPOverall(uint32 octets) {m_statistic.eMuleInOverallOctets += octets;}
#else
	void AddeMuleOutUDPOverall(uint32 octets) {m_statistic.eMuleOutOverallOctets += (octets + 20 /* IP */ + 8 /* UDP */);}
	void AddeMuleOutTCPOverall(uint32 octets) {m_statistic.eMuleOutOverallOctets += (octets + 20 /* IP */ + 20 /* TCP */);}
	void AddeMuleInUDPOverall(uint32 octets) {m_statistic.eMuleInOverallOctets += (octets + 20 /* IP */ + 8 /* UDP */);}
	void AddeMuleInTCPOverall(uint32 octets) {m_statistic.eMuleInOverallOctets += (octets + 20 /* IP */ + 20 /* TCP */);}
#endif

	// Accessor, used for the control of the bandwidth (=> slope)
	uint64 GeteMuleOut() const {return m_statistic.eMuleOutOctets;}
	uint64 GeteMuleIn() const {return m_statistic.eMuleInOctets;}
	uint64 GeteMuleOutOverall() const {return m_statistic.eMuleOutOverallOctets;}
	uint64 GeteMuleInOverall() const {return m_statistic.eMuleInOverallOctets;}
   	uint64 GetNetworkOut() const {return m_statistic.networkOutOctets;} // [TPT] - Xman
   	uint64 GetNetworkIn() const {return m_statistic.networkInOctets;} // [TPT] - Xman
	uint32 GetStartTick() const {return m_statistic.timeStamp;}

	// Retrieve datarates for barline + upload slot management + Graphic
	void GetDatarates(uint8 samples, 
					  uint32& eMuleIn, uint32& eMuleInOverall, 
					  uint32& eMuleOut, uint32& eMuleOutOverall,
					  uint32& networkIn, uint32& networkOut) const;

	// Retrieve datarates Graphic
	void GetFullHistoryDatarates(uint32& eMuleInHistory, uint32& eMuleOutHistory, 
								 uint32& eMuleInSession, uint32& eMuleOutSession) const;

	// Check if the NAFC is available on this computer
	bool IsNAFCAvailable() const {return (m_fGetIfEntry != NULL);}

	// Full NAFC bandwidth control    
	float GetMaxDownload() const {return m_maxDownloadLimit;}
    float GetMaxUpload() const {return m_maxUploadLimit;}

private:
	// Adapter access
	DWORD getAdapterIndex();
	DWORD m_currentAdapterIndex;

	// Type definition
	#pragma pack(1)
	struct Statistic {
		Statistic() {}
		Statistic(const Statistic& ref, uint32 time)
		:	eMuleOutOctets(ref.eMuleOutOctets),
			eMuleInOctets(ref.eMuleInOctets),
			eMuleOutOverallOctets(ref.eMuleOutOverallOctets),
			eMuleInOverallOctets(ref.eMuleInOverallOctets),
			networkOutOctets(ref.networkOutOctets),
			networkInOctets(ref.networkInOctets),
			timeStamp(time) {}

		uint64 eMuleOutOctets; // Data
		uint64 eMuleInOctets;
		uint64 eMuleOutOverallOctets; // Data+Control
		uint64 eMuleInOverallOctets;
      		uint64 networkOutOctets; // DataFlow of the network Adapter // [TPT] - Xman
      		uint64 networkInOctets; // [TPT] - Xman
		uint32 timeStamp; // Use a time stamp to compensate the inaccuracy of the timer (based on enkeyDEV(Ottavio84))
	};
	#pragma pack()
	typedef CList<Statistic> StatisticHistory; // Use MS container for its memory management

	StatisticHistory m_statisticHistory; // History for the graphic
	Statistic m_statistic; // Current value

    float m_maxDownloadLimit; // Used for auto U/D limits
    float m_maxUploadLimit; 

    // Keep last result to detect an overflow
    DWORD m_networkOutOctets; // [TPT] - Xman
    DWORD m_networkInOctets; // [TPT] - Xman

    bool m_errorTraced;

	// Dynamic access to the iphlpapi.dll
	typedef DWORD (WINAPI *GETNUMBEROFINTERFACES)(PDWORD pdwNumIf);
	typedef DWORD (WINAPI *GETIFTABLE)(PMIB_IFTABLE pIfTable, PULONG pdwSize, BOOL bOrder);
    typedef DWORD (WINAPI *GETIPADDRTABLE)(PMIB_IPADDRTABLE pIpAddrTable, PULONG pdwSize, BOOL bOrder);
    typedef DWORD (WINAPI *GETIFENTRY)(PMIB_IFROW pIfRow);

	HINSTANCE m_hIphlpapi;
	GETIFTABLE m_fGetIfTable;
    GETIPADDRTABLE m_fGetIpAddrTable;
	GETIFENTRY m_fGetIfEntry;
	GETNUMBEROFINTERFACES m_fGetNumberOfInterfaces;

private:
	// Don't allow canonical behavior
	CBandWidthControl(const CBandWidthControl&);
	CBandWidthControl& operator=(const CBandWidthControl&);
};